@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -292,6 +292,7 @@ main(argc, argv)
 		case arglex_token_string:
 			if (source)
 			{
+				too_many_sources:
 				error_intl
 				(
 					0,
@@ -302,6 +303,13 @@ main(argc, argv)
 			source = arglex_value.alv_string;
 			break;
 
+		case arglex_token_stdio:
+			if (source)
+				goto too_many_sources;
+			source = "-";
+			cache = 0;
+			break;
+
 		case arglex_token_absent_local_ignore:
 			if (option.o_absent_local != -1)
 			{
