@@ -30,6 +30,7 @@
 #include <error_intl.h>
 #include <flatten.h>
 #include <input/file_text.h>
+#include <input/stdin.h>
 #include <mem.h>
 #include <os_interface.h>
 #include <sniff.h>
@@ -513,8 +514,11 @@ sniffer(filename, prnam, result)
 	struct stat	st;
 	size_t		j;
 	sub_context_ty	*scp;
+	static string_ty *dash;
 
 	trace(("sniffer(filename = \"%s\")\n{\n"/*}*/, filename->str_text));
+	if (!dash)
+		dash = str_from_c("-");
 	if (prnam)
 	{
 		print_without_prefix(filename, result);
@@ -527,7 +531,9 @@ sniffer(filename, prnam, result)
 	 */
 	cp = cache_search(filename);
 	assert(cp);
-	if (stat(filename->str_text, &st) < 0)
+	if (str_equal(filename, dash))
+		memset(&st, -1, sizeof(st));
+	else if (stat(filename->str_text, &st) < 0)
 	{
 		/*
 		 * here for failure to open/find a file
@@ -588,9 +594,14 @@ sniffer(filename, prnam, result)
 		verbose_intl(scp, i18n("cache miss for \"$filename\" file"));
 		sub_context_delete(scp);
 
-		if (!os_exists(filename->str_text))
-			goto absent;
-		fp = input_file_text_open(filename);
+		if (str_equal(dash, filename))
+			fp = input_stdin();
+		else
+		{
+			if (!os_exists(filename->str_text))
+				goto absent;
+			fp = input_file_text_open(filename);
+		}
 		string_list_constructor(&type1);
 		string_list_constructor(&type2);
 		if (lang->scan(fp, &type1, &type2))
@@ -891,34 +902,40 @@ sniff(filename)
 	if (term_hold != SIG_IGN)
 		signal(SIGTERM, interrupt);
 
-	assert(filename);
-	trace(("sniff(filename = \"%s\")\n{\n"/*}*/, filename));
-	if (!os_exists(filename))
+	if (0 == strcmp(filename, "-"))
 	{
-		switch (option.o_absent_program)
+		s = str_from_c("-");
+	}
+	else
+	{
+		trace(("sniff(filename = \"%s\")\n{\n", filename));
+		if (!os_exists(filename))
 		{
-		case absent_error:
-			scp = sub_context_new();
-			sub_errno_setx(scp, ENOENT);
-			sub_var_set(scp, "File_Name", "%s", filename);
-			fatal_intl(scp, i18n("open $filename: $errno"));
-			/* NOTREACHED */
-			break;
-
-		case absent_mention:
-			scp = sub_context_new();
-			sub_errno_setx(scp, ENOENT);
-			sub_var_set(scp, "File_Name", "%s", filename);
-			error_intl(scp, i18n("open $filename: $errno"));
-			/* NOTREACHED */
-			break;
-
-		default:
-			break;
+			switch (option.o_absent_program)
+			{
+			case absent_error:
+				scp = sub_context_new();
+				sub_errno_setx(scp, ENOENT);
+				sub_var_set(scp, "File_Name", "%s", filename);
+				fatal_intl(scp, i18n("open $filename: $errno"));
+				/* NOTREACHED */
+				break;
+	
+			case absent_mention:
+				scp = sub_context_new();
+				sub_errno_setx(scp, ENOENT);
+				sub_var_set(scp, "File_Name", "%s", filename);
+				error_intl(scp, i18n("open $filename: $errno"));
+				/* NOTREACHED */
+				break;
+	
+			default:
+				break;
+			}
+			goto done;
 		}
-		goto done;
+		s = str_from_c(filename);
 	}
-	s = str_from_c(filename);
 
 	string_list_constructor(&result);
 	sniffer(s, 0, &result);
@@ -944,7 +961,7 @@ sniff(filename)
 	signal(SIGINT, int_hold);
 	signal(SIGHUP, hup_hold);
 	signal(SIGTERM, term_hold);
-	trace((/*{*/"}\n"));
+	trace(("}\n"));
 }
 
 
