@@ -50,7 +50,7 @@ static char	**argv;
 arglex_value_ty	arglex_value;
 arglex_token_ty	arglex_token;
 static arglex_table_ty *utable;
-static	char	*partial;
+static const char *partial;
 
 
 static char *base_name _((char *));
@@ -165,12 +165,12 @@ arglex_init(ac, av, tp)
 
 int
 arglex_compare(formal, actual)
-	char	*formal;
-	char	*actual;
+	const char	*formal;
+	const char	*actual;
 {
-	char	fc;
-	char	ac;
-	int	result;
+	char		fc;
+	char		ac;
+	int		result;
 
 	trace(("arglex_compare(formal = \"%s\", actual = \"%s\")\n{\n",
 		formal, actual));
@@ -531,7 +531,7 @@ arglex()
 
 	case 1:
 		if (partial)
-			pushback[pushback_depth++] = partial;
+			pushback[pushback_depth++] = (char *)partial;
 		arg = hit[0]->name;
 		arglex_token = hit[0]->token;
 		break;
