@@ -296,9 +296,13 @@ copy_a_string(s)
  */
 
 void
+#if defined(__STDC__) && __STDC__
+error_raw(char *s, ...)
+#else
 error_raw(s sva_last)
 	char		*s;
 	sva_last_decl
+#endif
 {
 	va_list		ap;
 	char		*msg;
@@ -334,9 +338,13 @@ error_raw(s sva_last)
 
 /*VARARGS1*/
 void
+#if defined(__STDC__) && __STDC__
+nerror_raw(char *s, ...)
+#else
 nerror_raw(s sva_last)
 	char		*s;
 	sva_last_decl
+#endif
 {
 	va_list		ap;
 	int		n;
@@ -378,9 +386,13 @@ nerror_raw(s sva_last)
 
 /*VARARGS1*/
 void
+#if defined(__STDC__) && __STDC__
+nfatal_raw(char *s, ...)
+#else
 nfatal_raw(s sva_last)
 	char		*s;
 	sva_last_decl
+#endif
 {
 	va_list		ap;
 	int		n;
@@ -421,9 +433,13 @@ nfatal_raw(s sva_last)
  */
 
 void
+#if defined(__STDC__) && __STDC__
+fatal_raw(char *s, ...)
+#else
 fatal_raw(s sva_last)
 	char		*s;
 	sva_last_decl
+#endif
 {
 	va_list		ap;
 	char		*msg;
