@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,8 +43,8 @@ extern string_ty *str_true;
 extern string_ty *str_false;
 
 void str_initialize _((void));
-string_ty *str_from_c _((char *));
-string_ty *str_n_from_c _((char *, size_t));
+string_ty *str_from_c _((const char *));
+string_ty *str_n_from_c _((const char *, size_t));
 string_ty *str_copy _((string_ty *));
 void str_free _((string_ty *));
 string_ty *str_catenate _((string_ty *, string_ty *));
@@ -59,8 +59,8 @@ int str_valid _((string_ty *));
 #endif
 
 string_ty *str_field _((string_ty *s, int sep, int fldnum));
-string_ty *str_format _((char *, ...));
-string_ty *str_vformat _((char *, va_list));
+string_ty *str_format _((const char *, ...));
+string_ty *str_vformat _((const char *, va_list));
 
 #define str_equal(s1, s2) ((s1) == (s2))
 
