@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997, 1998, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -260,6 +260,88 @@ string_list_copy_constructor(to, from)
 }
 
 
+/*
+ * NAME
+ *	wl2str_respect_empty - form a string from a word list
+ *
+ * SYNOPSIS
+ *	string_ty *wl2str_respect_empty(string_list_ty *wlp, int start,
+ *		int stop, char *sep, int empty);
+ *
+ * DESCRIPTION
+ *	Wl2str is used to form a string from a word list.  The ``empty''
+ *	argument is true if empty strings should be honoured; if false
+ *	empty strings will be ignored.
+ *
+ * RETURNS
+ *	A pointer to the newly formed string in dynamic memory.
+ *
+ * CAVEAT
+ *	It is the responsibility of the caller to ensure that the
+ *	new string is freed when finished with, by a call to str_free().
+ */
+
+string_ty *
+wl2str_respect_empty(wl, start, stop, sep, empty)
+	const string_list_ty *wl;
+	int		start;
+	int		stop;
+	const char	*sep;
+	int		empty;
+{
+	int		j;
+	static char	*tmp;
+	static size_t	tmplen;
+	size_t		length;
+	size_t		seplen;
+	char		*pos;
+	string_ty	*s;
+
+	if (!sep)
+		sep = " ";
+	seplen = strlen(sep);
+	length = 0;
+	for (j = start; j <= stop && j < wl->nstrings; j++)
+	{
+		s = wl->string[j];
+		if (s->str_length || empty)
+		{
+			if (length > 0 || (empty && j > start))
+				length += seplen;
+			length += s->str_length;
+		}
+	}
+
+	if (tmplen < length)
+	{
+		tmplen = length;
+		tmp = mem_change_size(tmp, tmplen);
+	}
+
+	pos = tmp;
+	for (j = start; j <= stop && j < wl->nstrings; j++)
+	{
+		s = wl->string[j];
+		if (s->str_length || empty)
+		{
+			if (seplen && (pos != tmp || (empty && j > start)))
+			{
+				memcpy(pos, sep, seplen);
+				pos += seplen;
+			}
+			if (s->str_length > 0)
+			{
+				memcpy(pos, s->str_text, s->str_length);
+				pos += s->str_length;
+			}
+		}
+	}
+
+	s = str_n_from_c(tmp, length);
+	return s;
+}
+
+
 /*
  * NAME
  *	wl2str - form a string from a word list
@@ -283,55 +365,9 @@ wl2str(wl, start, stop, sep)
 	const string_list_ty *wl;
 	int		start;
 	int		stop;
-	char		*sep;
+	const char	*sep;
 {
-	int		j;
-	static char	*tmp;
-	static size_t	tmplen;
-	size_t		length;
-	size_t		seplen;
-	char		*pos;
-	string_ty	*s;
-
-	if (!sep)
-		sep = " ";
-	seplen = strlen(sep);
-	length = 0;
-	for (j = start; j <= stop && j < wl->nstrings; j++)
-	{
-		s = wl->string[j];
-		if (s->str_length)
-		{
-			if (length)
-				length += seplen;
-			length += s->str_length;
-		}
-	}
-
-	if (tmplen < length)
-	{
-		tmplen = length;
-		tmp = mem_change_size(tmp, tmplen);
-	}
-
-	pos = tmp;
-	for (j = start; j <= stop && j < wl->nstrings; j++)
-	{
-		s = wl->string[j];
-		if (s->str_length)
-		{
-			if (pos != tmp)
-			{
-				memcpy(pos, sep, seplen);
-				pos += seplen;
-			}
-			memcpy(pos, s->str_text, s->str_length);
-			pos += s->str_length;
-		}
-	}
-
-	s = str_n_from_c(tmp, length);
-	return s;
+	return wl2str_respect_empty(wl, start, stop, sep, 0);
 }
 
 
