@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,9 @@ struct	string_list_ty
 
 int string_list_member _((const string_list_ty *, string_ty *));
 int string_list_intersect _((const string_list_ty *, const string_list_ty *));
-string_ty *wl2str _((const string_list_ty *, int, int, char *));
+string_ty *wl2str_respect_empty _((const string_list_ty *, int, int,
+	const char *, int));
+string_ty *wl2str _((const string_list_ty *, int, int, const char *));
 void str2wl _((string_list_ty *, string_ty *, char *, int));
 void string_list_append _((string_list_ty *, string_ty *));
 void string_list_append_list _((string_list_ty *, const string_list_ty *));
