@@ -57,7 +57,7 @@ typedef wstring_ty *(*fp)_((sub_context_ty *, wstring_list_ty *));
 typedef struct table_ty table_ty;
 struct table_ty
 {
-	char		*name;
+	const char	*name;
 	fp		func;
 	int		resubstitute;
 	wstring_ty	*value;
@@ -1048,7 +1048,7 @@ subst(scp, s)
 wstring_ty *
 subst_intl_wide(scp, msg)
 	sub_context_ty	*scp;
-	char		*msg;
+	const char	*msg;
 {
 	char		*tmp;
 	wstring_ty	*s;
@@ -1077,7 +1077,7 @@ subst_intl_wide(scp, msg)
 string_ty *
 subst_intl(scp, s)
 	sub_context_ty	*scp;
-	char		*s;
+	const char	*s;
 {
 	wstring_ty	*result_wide;
 	string_ty	*result;
@@ -1193,11 +1193,15 @@ sub_var_clear(scp)
  */
 
 void
+#if defined(__STDC__) && __STDC__
+sub_var_set(sub_context_ty *scp, const char *name, const char *fmt, ...)
+#else
 sub_var_set(scp, name, fmt sva_last)
 	sub_context_ty	*scp;
-	char		*name;
-	char		*fmt;
+	const char	*name;
+	const char	*fmt;
 	sva_last_decl
+#endif
 {
 	va_list		ap;
 	string_ty	*s;
@@ -1234,7 +1238,7 @@ sub_var_set(scp, name, fmt sva_last)
 void
 sub_var_resubstitute(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1252,7 +1256,7 @@ sub_var_resubstitute(scp, name)
 void
 sub_var_override(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1270,7 +1274,7 @@ sub_var_override(scp, name)
 void
 sub_var_optional(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
@@ -1288,7 +1292,7 @@ sub_var_optional(scp, name)
 void
 sub_var_append_if_unused(scp, name)
 	sub_context_ty	*scp;
-	char		*name;
+	const char	*name;
 {
 	table_ty	*the_end;
 	table_ty	*svp;
