@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,9 +32,9 @@
 #include <builtin/text.h>
 #include <error_intl.h>
 #include <expr/position.h>
-#include <mem.h>
 #include <quote.h>
 #include <str_list.h>
+#include <stracc.h>
 #include <trace.h>
 
 
@@ -59,8 +59,11 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_upcase(result, args, pp, ocp)
+static int upcase_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+upcase_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -72,21 +75,6 @@ builtin_upcase(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty *s;
@@ -99,6 +87,14 @@ builtin_upcase(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_upcase =
+{
+	"upcase",
+	upcase_interpret,
+	upcase_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_downcase - downcase strings
@@ -120,8 +116,11 @@ builtin_upcase(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_downcase(result, args, pp, ocp)
+static int downcase_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+downcase_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -133,21 +132,6 @@ builtin_downcase(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
@@ -160,6 +144,14 @@ builtin_downcase(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_downcase =
+{
+	"downcase",
+	downcase_interpret,
+	downcase_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_prepost - add prefix and suffix
@@ -182,8 +174,11 @@ builtin_downcase(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_prepost(result, args, pp, ocp)
+static int prepost_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+prepost_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -228,6 +223,14 @@ builtin_prepost(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_prepost =
+{
+	"prepost",
+	prepost_interpret,
+	prepost_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_head - head of a wordlist
@@ -249,8 +252,11 @@ builtin_prepost(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_head(result, args, pp, ocp)
+static int head_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+head_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -266,6 +272,21 @@ builtin_head(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_head =
+{
+	"head",
+	head_interpret,
+	head_interpret, /* script */
+};
+
+builtin_ty builtin_firstword =
+{
+	"firstword",
+	head_interpret,
+	head_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_tail - tail of a wordlist
@@ -288,8 +309,11 @@ builtin_head(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_tail(result, args, pp, ocp)
+static int tail_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+tail_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -307,6 +331,14 @@ builtin_tail(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_tail =
+{
+	"tail",
+	tail_interpret,
+	tail_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_catenate - catenate a wordlist
@@ -328,18 +360,19 @@ builtin_tail(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_catenate(result, args, pp, ocp)
+static int
+catenate_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+catenate_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
 	const struct opcode_context_ty *ocp;
 {
+	static stracc	sa;
 	int		j;
-	static char	*tmp;
-	static size_t	tmplen;
-	size_t		length;
-	char		*pos;
 	string_ty	*s;
 
 	trace(("catenate\n"));
@@ -354,38 +387,27 @@ builtin_catenate(result, args, pp, ocp)
 		return 0;
 	}
 
-	length = 0;
-	for (j = 1; j < args->nstrings; j++)
-		length += args->string[j]->str_length;
-	if (!tmp)
-	{
-		tmplen = length;
-		if (tmplen < 16)
-			tmplen = 16;
-		tmp = mem_alloc(tmplen);
-	}
-	else
-	{
-		if (tmplen < length)
-		{
-			tmplen = length;
-			tmp = mem_change_size(tmp, tmplen);
-		}
-	}
-	pos = tmp;
+	sa_open(&sa);
 	for (j = 1; j < args->nstrings; j++)
 	{
 		s = args->string[j];
-		memcpy(pos, s->str_text, s->str_length);
-		pos += s->str_length;
+		sa_chars(&sa, s->str_text, s->str_length);
 	}
-	s = str_n_from_c(tmp, length);
+	s = sa_close(&sa);
 	string_list_append(result, s);
 	str_free(s);
 	return 0;
 }
 
 
+builtin_ty builtin_catenate =
+{
+	"catenate",
+	catenate_interpret,
+	catenate_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_count - length of a word list
@@ -407,8 +429,11 @@ builtin_catenate(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_count(result, args, pp, ocp)
+static int count_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+count_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -427,6 +452,21 @@ builtin_count(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_count =
+{
+	"count",
+	count_interpret,
+	count_interpret, /* script */
+};
+
+builtin_ty builtin_words =
+{
+	"words",
+	count_interpret,
+	count_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_quote - quote the arguments
@@ -448,8 +488,11 @@ builtin_count(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_quote(result, args, pp, ocp)
+static int quote_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+quote_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -472,6 +515,14 @@ builtin_quote(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_quote =
+{
+	"quote",
+	quote_interpret,
+	quote_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_sort - sort the arguments
@@ -509,8 +560,11 @@ cmp(va, vb)
 }
 
 
-int
-builtin_sort(result, args, pp, ocp)
+static int sort_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+sort_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -546,3 +600,11 @@ builtin_sort(result, args, pp, ocp)
 	);
 	return 0;
 }
+
+
+builtin_ty builtin_sort =
+{
+	"sort",
+	sort_interpret,
+	sort_interpret, /* script */
+};
