@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,8 +43,12 @@ enum flag_value_ty
 	RF_FINGERPRINT_OFF,
 	RF_FORCE,
 	RF_FORCE_OFF,
+	RF_GATEFIRST,
+	RF_GATEFIRST_OFF,
 	RF_IMPLICIT_ALLOWED,
 	RF_IMPLICIT_ALLOWED_OFF,
+	RF_INCLUDE_COOKED_WARNING,
+	RF_INCLUDE_COOKED_WARNING_OFF,
 	RF_METER,
 	RF_METER_OFF,
 	RF_MKDIR,
@@ -64,6 +68,8 @@ enum flag_value_ty
 	RF_UPDATE,
 	RF_UPDATE_MAX,
 	RF_UPDATE_OFF,
+	RF_MATCH_MODE_COOK,
+	RF_MATCH_MODE_REGEX,
 	RF_max	/* MUST be last */
 };
 typedef enum flag_value_ty flag_value_ty;
