@@ -41,7 +41,7 @@
 #include <opcode/context.h>
 #include <opcode/list.h>
 #include <option.h>
-#include <os.h>
+#include <os_interface.h>
 #include <recipe.h>
 #include <recipe/list.h>
 #include <star.h>
@@ -162,12 +162,14 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	sub_context_ty	*scp;
 	cascade_list_ty	cascade;
 	int		cascade_enabled;
+	opcode_context_ty *ocp;
 
 	/*
 	 * be optimistic, assume success
 	 */
 	trace(("graph_check_ingredients(gp = %8.8lX, rp = %8.8lX, preference = %d)\n{\n"/*}*/,
 		(long)gp, (long)rp, (int)preference));
+	ocp = 0;
 	cascade_list_constructor(&cascade);
 	if (rp->inhibit)
 	{
@@ -214,7 +216,8 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	 * define the [targets] variable.
 	 */
 	assert(target->nstrings);
-	id_assign_push(id_targets, id_variable_new(target));
+	ocp = opcode_context_new(0, mp);
+	opcode_context_id_assign(ocp, id_targets, id_variable_new(target), -1);
 
 	/*
 	 * Define the [target] variable to be the first target in the list.
@@ -223,7 +226,7 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	wlp2 = 0;
 	wlp1 = string_list_new();
 	string_list_append(wlp1, target1);
-	id_assign_push(id_target, id_variable_new(wlp1));
+	opcode_context_id_assign(ocp, id_target, id_variable_new(wlp1), -1);
 	string_list_delete(wlp1);
 
 	/*
@@ -256,7 +259,8 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	if (rp->precondition && option_test(OPTION_GATEFIRST))
 	{
 		int		flag;
-		flag = opcode_list_run_bool(rp->precondition, mp);
+
+		flag = opcode_context_run_bool(ocp, rp->precondition);
 
 		switch (flag)
 		{
@@ -289,7 +293,7 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	 *	the second set must be cookable
 	 */
 	trace(("check first ingredients\n"));
-	wlp1 = opcode_list_run(rp->need1, mp);
+	wlp1 = opcode_context_run(ocp, rp->need1);
 	if (!wlp1)
 	{
 		error:
@@ -423,7 +427,7 @@ backtrack:
 	}
 
 	trace(("check second ingredients\n"));
-	wlp2 = opcode_list_run(rp->need2, mp);
+	wlp2 = opcode_context_run(ocp, rp->need2);
 	if (!wlp2)
 	{
 		string_list_delete(wlp1);
@@ -680,13 +684,12 @@ backtrack:
 		 * variable.  (Note: can't define [younger] until we
 		 * walk the graph.)
 		 */
-		id_assign_push(id_need, id_variable_new(&need));
+		opcode_context_id_assign(ocp, id_need, id_variable_new(&need), -1);
 
 		/*
 		 * evaluate the predicate
 		 */
-		flag = opcode_list_run_bool(rp->precondition, mp);
-		id_unassign(id_need);
+		flag = opcode_context_run_bool(ocp, rp->precondition);
 
 		/*
 		 * act on the results of the predicate
@@ -773,9 +776,9 @@ done:
 	}
 	trace(("target->nstrings = %ld;\n", (long)target->nstrings));
 	trace(("need_gfl->nfiles = %ld;\n", (long)need_gfl->nfiles));
-	id_unassign(id_target);
-	id_unassign(id_targets);
 	star_as_specified('*');
+	if (ocp)
+		opcode_context_delete(ocp);
 	trace(("return %s;\n", graph_build_status_name(status)));
 	trace((/*{*/"}\n"));
 	return status;
@@ -825,9 +828,11 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	graph_recipe_ty	*grp;
 	string_list_ty	target;
 	size_t		k;
+	opcode_context_ty *ocp;
 
 	trace(("graph_check_recipe(gp = %8.8lX, rp = %8.8lX, preference = %d)\n{\n",
 		(long)gp, (long)rp, (int)preference));
+	ocp = 0;
 	status =
 		graph_check_ingredients
 		(
@@ -854,9 +859,7 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 		 */
 		if (mp)
 			match_delete(mp);
-		trace(("return %s;\n", graph_build_status_name(status)));
-		trace(("}\n"));
-		return status;
+		goto done;
 
 	case graph_build_status_success:
 		/*
@@ -878,9 +881,10 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	/*
 	 * Calculate the single_thread values.
 	 */
+	ocp = opcode_context_new(0, mp);
 	if (rp->single_thread)
 	{
-		grp->single_thread = opcode_list_run(rp->single_thread, mp);
+		grp->single_thread = opcode_context_run(ocp, rp->single_thread);
 		if (!grp->single_thread)
 			status = graph_build_status_error;
 		else if (grp->single_thread->nstrings == 0)
@@ -895,7 +899,7 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	 */
 	if (rp->host_binding)
 	{
-		grp->host_binding = opcode_list_run(rp->host_binding, mp);
+		grp->host_binding = opcode_context_run(ocp, rp->host_binding);
 		if (!grp->host_binding)
 			status = graph_build_status_error;
 		else if (grp->host_binding->nstrings == 0)
@@ -947,6 +951,9 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	/*
 	 * return result
 	 */
+	done:
+	if (ocp)
+		opcode_context_delete(ocp);
 	trace(("return %s;\n", graph_build_status_name(status)));
 	trace(("}\n"));
 	return status;
@@ -1042,6 +1049,7 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 	const recipe_list_ty *explicit_recipes;
 	sub_context_ty	*scp;
 	leaf_ness_ty	leaf;
+	opcode_context_ty *ocp;
 
 	/*
 	 * The gp->try_list is a list of files that were not used, and
@@ -1947,11 +1955,12 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			}
 			trace(("leaf\n"));
 
+			ocp = opcode_context_new(0, 0);
 			if
 			(
 				leaf == leaf_ness_leaf_explicit
 			&&
-				cook_mtime_oldest(target, (long *)0, 32767L) == 0
+				cook_mtime_oldest(ocp, target, (long *)0, 32767L) == 0
 			)
 			{
 				/*
@@ -1966,8 +1975,10 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			     i18n("\"$filename\" primary source file not found")
 				);
 				sub_context_delete(scp);
+				opcode_context_delete(ocp);
 				goto bomb_with_error;
 			}
+			opcode_context_delete(ocp);
 
 			result->status = graph_build_status_success;
 			result->gfp = gfp;
