@@ -25,6 +25,7 @@
 #include <graph/file.h>
 #include <graph/file_list.h>
 #include <graph/recipe.h>
+#include <opcode/context.h>
 #include <option.h>
 #include <recipe.h>
 #include <str_list.h>
@@ -62,6 +63,7 @@ graph_recipe_check(grp, gp)
 	int		up_to_date;
 	time_t		need_age;
 	size_t		j;
+	opcode_context_ty *ocp;
 
 	trace(("graph_recipe_check(grp = %08lX)\n{\n"/*}*/, (long)grp));
 	status = graph_walk_status_uptodate;
@@ -87,6 +89,7 @@ graph_recipe_check(grp, gp)
 	 */
 	target_age = 0;
 	target_depth = 32767;
+	ocp = opcode_context_new(0, 0);
 	for (j = 0; j < grp->output->nfiles; ++j)
 	{
 		graph_file_ty	*gfp2;
@@ -96,7 +99,7 @@ graph_recipe_check(grp, gp)
 		gfp2 = grp->output->file[j];
 		
 		depth2 = 32767;
-		age2 = cook_mtime_newest(gfp2->filename, &depth2, depth2);
+		age2 = cook_mtime_newest(ocp, gfp2->filename, &depth2, depth2);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -123,7 +126,7 @@ graph_recipe_check(grp, gp)
 
 		gfp2 = grp->input->file[j];
 		depth2 = 32767;
-		age2 = cook_mtime_oldest(gfp2->filename, &depth2, depth2);
+		age2 = cook_mtime_oldest(ocp, gfp2->filename, &depth2, depth2);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -183,6 +186,7 @@ graph_recipe_check(grp, gp)
 	 * cancel the recipe flags
 	 */
 ret:
+	opcode_context_delete(ocp);
 	option_undo_level(OPTION_LEVEL_RECIPE);
 
 	trace(("return %s;\n", graph_walk_status_name(status)));
