@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -37,48 +37,13 @@
 #include <opcode/context.h>
 #include <opcode/list.h>
 #include <option.h>
-#include <os.h>
+#include <os_interface.h>
 #include <recipe.h>
 #include <stmt.h>
 #include <str_list.h>
 #include <trace.h>
 
 
-/*
- * NAME
- *	run_it
- *
- * SYNOPSIS
- *	opcode_status_ty run_it(opcode_list_ty *);
- *
- * DESCRIPTION
- *	The run_it function is used to execute a pre-compiled opcode
- *	stream.  Use to run recipe bodies.
- *
- * RETURNS
- *	opcode_status_ty - indicating success or failure
- */
-
-static opcode_status_ty run_it _((graph_ty *gp, opcode_list_ty *,
-	const match_ty *));
-
-static opcode_status_ty
-run_it(gp, olp, mp)
-	graph_ty	*gp;
-	opcode_list_ty	*olp;
-	const match_ty	*mp;
-{
-	opcode_context_ty *ocp;
-	opcode_status_ty status;
-
-	ocp = opcode_context_new(olp, mp);
-	ocp->gp = gp;
-	status = opcode_context_execute_nowait(ocp);
-	opcode_context_delete(ocp);
-	return status;
-}
-
-
 static int make_target_directories _((graph_file_list_nrc_ty *));
 
 static int
@@ -111,10 +76,12 @@ make_target_directories(gflp)
 }
 
 
-static string_ty *host_binding_round_robin _((string_list_ty  *));
+static string_ty *host_binding_round_robin _((opcode_context_ty *,
+	string_list_ty  *));
 
 static string_ty *
-host_binding_round_robin(slp)
+host_binding_round_robin(ocp, slp)
+	opcode_context_ty *ocp;
 	string_list_ty	*slp;
 {
 	static int	j;
@@ -128,7 +95,7 @@ host_binding_round_robin(slp)
 
 		if (!key)
 			key = str_from_c("parallel_hosts");
-		idp = id_search(key);
+		idp = opcode_context_id_search(ocp, key);
 		if (!idp)
 			return 0;
 		slp = id_variable_query2(idp);
@@ -191,6 +158,12 @@ graph_recipe_run(grp, gp)
 	need_age = 0;
 	phony = !grp->rp->out_of_date;
 
+	/*
+	 * create the opcode context for this recipe
+	 */
+	grp->ocp = opcode_context_new(0, grp->mp);
+	grp->ocp->gp = gp;
+
 	/*
 	 * Warn about essential information which is kept only in
 	 * derived files.
@@ -226,7 +199,7 @@ graph_recipe_run(grp, gp)
 	}
 	else
 		target1 = str_from_c("\7bogus\7"); /* mem leak */
-	id_assign_push(id_target, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_target, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -236,7 +209,7 @@ graph_recipe_run(grp, gp)
 	assert(grp->input);
 	for (j = 0; j < grp->output->nfiles; ++j)
 		string_list_append(&wl, grp->output->file[j]->filename);
-	id_assign_push(id_targets, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_targets, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -246,7 +219,7 @@ graph_recipe_run(grp, gp)
 	assert(grp->input);
 	for (j = 0; j < grp->input->nfiles; ++j)
 		string_list_append(&wl, grp->input->file[j]->filename);
-	id_assign_push(id_need, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_need, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -255,7 +228,7 @@ graph_recipe_run(grp, gp)
 	 * times.  [[Fake an assignment to avoid problems with errors.]]
 	 */
 	string_list_constructor(&younger);
-	id_assign_push(id_younger, id_variable_new(&younger));
+	opcode_context_id_assign(grp->ocp, id_younger, id_variable_new(&younger), -1);
 
 	/*
 	 * Flags apply to the precondition and to the ingredients
@@ -267,7 +240,7 @@ graph_recipe_run(grp, gp)
 	/*
 	 * see of the recipe is forced to activate
 	 */
-	forced = option_test(OPTION_FORCE);
+	forced = option_test(OPTION_FORCE) | grp->multi_forced;
 	if (forced && show_reasoning)
 	{
 		scp = sub_context_new();
@@ -309,7 +282,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 		if (option_test(OPTION_FINGERPRINT))
 		{
 			depth2 = 32767;
-			age2 = cook_mtime_oldest(gfp2->filename, &depth2, depth2);
+			age2 = cook_mtime_oldest(grp->ocp, gfp2->filename, &depth2, depth2);
 			if (age2 < 0)
 			{
 				/* error message already printed */
@@ -320,7 +293,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 		}
 		
 		depth2 = 32767;
-		age2 = cook_mtime_newest(gfp2->filename, &depth2, depth2);
+		age2 = cook_mtime_newest(grp->ocp, gfp2->filename, &depth2, depth2);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -391,7 +364,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 
 		gfp2 = grp->input->file[j];
 		depth2 = 32767;
-		age2 = cook_mtime_oldest(gfp2->filename, &depth2, target_depth+1);
+		age2 = cook_mtime_oldest(grp->ocp, gfp2->filename, &depth2, target_depth+1);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -552,7 +525,7 @@ now younger (reason)")
 	 * Assign the ``younger'' variable.
 	 * (Use a normal assignment, we did the push already.)
 	 */
-	id_assign(id_younger, id_variable_new(&younger));
+	opcode_context_id_assign(grp->ocp, id_younger, id_variable_new(&younger), -1);
 	string_list_destructor(&younger);
 
 	/*
@@ -652,16 +625,11 @@ now younger (reason)")
 				 * run the recipe body
 				 */
 				trace(("doing it now\n"));
-				grp->ocp =
-					opcode_context_new
-					(
-						grp->rp->out_of_date,
-						grp->mp
-					);
-				grp->ocp->gp = gp;
+				opcode_context_call(grp->ocp, grp->rp->out_of_date);
 				hostname =
 					host_binding_round_robin
 					(
+						grp->ocp,
 						grp->host_binding
 					);
 				if (hostname)
@@ -699,8 +667,6 @@ now younger (reason)")
 					status = graph_walk_status_error;
 					break;
 				}
-				opcode_context_delete(grp->ocp);
-				grp->ocp = 0;
 
 				/*
 				 * Remove recipe targets on errors,
@@ -797,25 +763,18 @@ now younger (reason)")
 			 * about making it parallel.
 			 */
 			trace(("perform ``use'' clause\n"));
-			result = run_it(gp, grp->rp->up_to_date, grp->mp);
+			opcode_context_call(grp->ocp, grp->rp->up_to_date);
+			result = opcode_context_execute_nowait(grp->ocp);
 			if (result != opcode_status_success)
 				status = graph_walk_status_error;
 		}
 	}
 
-	/*
-	 * Delete the variables unique to recipe body execution.
-	 */
-ret:
-	id_unassign(id_target);
-	id_unassign(id_targets);
-	id_unassign(id_need);
-	id_unassign(id_younger);
-
 	/*
 	 * adjust file modification times
 	 * (tracking fingerprints as we go)
 	 */
+ret:
 	if (status == graph_walk_status_done && grp->rp->out_of_date)
 	{
 		time_t		mtime;
@@ -853,7 +812,7 @@ ret:
 					continue;
 				}
 				depth4 = 32767;
-				t = cook_mtime_oldest(gfp->filename, &depth4, depth4);
+				t = cook_mtime_oldest(grp->ocp, gfp->filename, &depth4, depth4);
 				if (t < 0)
 				{
 					status = graph_walk_status_error;
@@ -880,6 +839,7 @@ ret:
 							scp,
 					 i18n("$filename fingerprint unchanged")
 						);
+						sub_context_delete(scp);
 					}
 				}
 				else if (status != graph_walk_status_error)
@@ -954,7 +914,7 @@ ret:
 	
 			gfp2 = grp->input->file[j];
 			depth2 = 32767;
-			age2 = cook_mtime_newest(gfp2->filename, &depth2, depth2);
+			age2 = cook_mtime_newest(grp->ocp, gfp2->filename, &depth2, depth2);
 			if (age2 > need_age_youngest)
 				need_age_youngest = age2;
 		}
@@ -977,7 +937,7 @@ ret:
 
 			gfp = grp->output->file[j];
 			depth3 = 32767;
-			age3 = cook_mtime_newest(gfp->filename, &depth3, depth3);
+			age3 = cook_mtime_newest(grp->ocp, gfp->filename, &depth3, depth3);
 			if (age3 > 0 && depth3 == 0 && age3 < need_age_youngest)
 				os_mtime_adjust(gfp->filename, need_age_youngest);
 		}
@@ -987,6 +947,11 @@ ret:
 	 * cancel the recipe flags
 	 */
 	option_undo_level(OPTION_LEVEL_RECIPE);
+	if (grp->ocp)
+	{
+		opcode_context_delete(grp->ocp);
+		grp->ocp = 0;
+	}
 
 	trace(("return %s;\n", graph_walk_status_name(status)));
 	trace((/*{*/"}\n"));
