@@ -33,7 +33,7 @@
 #include <match.h>
 #include <opcode/context.h>
 #include <option.h>
-#include <os.h>
+#include <os_interface.h>
 #include <quote.h>
 #include <recipe.h>
 #include <star.h>
@@ -73,6 +73,9 @@ graph_recipe_script(grp, gp)
 	trace(("graph_recipe_script(grp = %08lX)\n{\n"/*}*/, (long)grp));
 	status = graph_walk_status_done;
 
+	grp->ocp = opcode_context_new(0, grp->mp);
+	grp->ocp->gp = gp;
+
 	/*
 	 * construct the ``target'' variable
 	 */
@@ -81,7 +84,7 @@ graph_recipe_script(grp, gp)
 	assert(grp->output->nfiles > 0);
 	if (grp->output->nfiles > 0)
 		string_list_append(&wl, grp->output->file[0]->filename);
-	id_assign_push(id_target, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_target, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -91,7 +94,7 @@ graph_recipe_script(grp, gp)
 	assert(grp->input);
 	for (j = 0; j < grp->output->nfiles; ++j)
 		string_list_append(&wl, grp->output->file[j]->filename);
-	id_assign_push(id_targets, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_targets, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -102,8 +105,8 @@ graph_recipe_script(grp, gp)
 	assert(grp->input);
 	for (j = 0; j < grp->input->nfiles; ++j)
 		string_list_append(&wl, grp->input->file[j]->filename);
-	id_assign_push(id_need, id_variable_new(&wl));
-	id_assign_push(id_younger, id_variable_new(&wl));
+	opcode_context_id_assign(grp->ocp, id_need, id_variable_new(&wl), -1);
+	opcode_context_id_assign(grp->ocp, id_younger, id_variable_new(&wl), -1);
 	string_list_destructor(&wl);
 
 	/*
@@ -237,15 +240,13 @@ graph_recipe_script(grp, gp)
 		}
 		else
 		{
-			opcode_context_ty *ocp;
 			opcode_status_ty status2;
 
 			trace(("doing it now\n"));
-			ocp = opcode_context_new(grp->rp->out_of_date, grp->mp);
-			status2 = opcode_context_script(ocp);
+			opcode_context_call(grp->ocp, grp->rp->out_of_date);
+			status2 = opcode_context_script(grp->ocp);
 			if (status2 != opcode_status_success)
 				status = graph_walk_status_error;
-			opcode_context_delete(ocp);
 		}
 	}
 
@@ -263,15 +264,13 @@ graph_recipe_script(grp, gp)
 	 */
 	if (grp->rp->up_to_date)
 	{
-		opcode_context_ty *ocp;
 		opcode_status_ty status2;
 
 		trace(("perform ``use'' clause\n"));
 		if (!forced)
 			printf("else\n");
-		ocp = opcode_context_new(grp->rp->up_to_date, grp->mp);
-		status2 = opcode_context_script(ocp);
-		opcode_context_delete(ocp);
+		opcode_context_call(grp->ocp, grp->rp->up_to_date);
+		status2 = opcode_context_script(grp->ocp);
 		if (status2 != opcode_status_success)
 			status = graph_walk_status_error;
 	}
@@ -282,18 +281,15 @@ graph_recipe_script(grp, gp)
 	if (!forced)
 		printf("fi\n");
 
-	/*
-	 * Delete the variables unique to recipe body execution.
-	 */
-	id_unassign(id_target);
-	id_unassign(id_targets);
-	id_unassign(id_need);
-	id_unassign(id_younger);
-
 	/*
 	 * cancel the recipe flags
 	 */
 	option_undo_level(OPTION_LEVEL_RECIPE);
+	if (grp->ocp)
+	{
+		opcode_context_delete(grp->ocp);
+		grp->ocp = 0;
+	}
 
 	star_as_specified('*');
 	trace(("return %s;\n", opcode_status_name(status)));
