@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,8 @@
 
 #include <error_intl.h>
 #include <listing.h>
-#include <os.h>
+#include <os_interface.h>
+#include <os/wait.h>
 #include <option.h>
 #include <quit.h>
 #include <trace.h>
@@ -151,7 +152,7 @@ log_close()
 			int who;
 			int status;
 
-			who = wait(&status);
+			who = os_waitpid(pid, &status);
 			if (who < 0 || who == pid)
 				break;
 		}
