@@ -24,6 +24,7 @@
 #include <ac/string.h>
 #include <ac/stdio.h>
 #include <ac/stdlib.h>
+#include <signal.h>
 
 #include <arglex.h>
 #include <builtin.h>
@@ -38,6 +39,7 @@
 #include <option.h>
 #include <parse.h>
 #include <star.h>
+#include <opcode/context.h>
 #include <progname.h>
 #include <quit.h>
 #include <trace.h>
@@ -568,9 +570,12 @@ static void
 set_command_line_goals()
 {
 	string_ty	*name;
+	opcode_context_ty *ocp;
 
 	name = str_from_c("command-line-goals");
-	id_assign(name, id_variable_new(&option.o_target));
+	ocp = opcode_context_new(0, 0);
+	opcode_context_id_assign(ocp, name, id_variable_new(&option.o_target), 0);
+	opcode_context_delete(ocp);
 	str_free(name);
 }
 
@@ -599,6 +604,20 @@ main(argc, argv)
 {
 	int		retval;
 
+	/*
+	 * Some versions of cron(8) and at(1) set SIGCHLD to SIG_IGN.
+	 * This is kinda dumb, because it breaks assumprions made in
+	 * libc (like pclose, for instance).  It also blows away most
+	 * of Cook's process handling.	We explicitly set the SIGCHLD
+	 * signal handling to SIG_DFL to make sure this signal does what
+	 * we expect no matter how we are invoked.
+	 */
+#ifdef SIGCHLD
+	signal(SIGCHLD, SIG_DFL);
+#else
+	signal(SIGCLD, SIG_DFL);
+#endif
+
 	/*
 	 * initialize things
 	 * (order is critical here)
@@ -665,6 +684,7 @@ main(argc, argv)
 			string_ty	*name;
 			string_ty	*value;
 			string_list_ty	wl;
+			opcode_context_ty *ocp;
 	
 			s = option.o_vardef.string[j]->str_text;
 			cp = strchr(s, '=');
@@ -675,7 +695,9 @@ main(argc, argv)
 			value = str_from_c(cp + 1);
 			str2wl(&wl, value, (char *)0, 0);
 			str_free(value);
-			id_assign(name, id_variable_new(&wl));
+			ocp = opcode_context_new(0, 0);
+			opcode_context_id_assign(ocp, name, id_variable_new(&wl), 0);
+			opcode_context_delete(ocp);
 			str_free(name);
 			string_list_destructor(&wl);
 		}
