@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1999 Peter Miller;
+ *	Copyright (C) 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -635,8 +635,21 @@ actual_begin = %08lX)\n{\n",
 		}
 		else
 			q = actual_end;
-		/* not allowed to be empty */
-		while (q > actual_begin)
+		/*
+                 * Pattern elements are not allowed to be empty at the
+                 * start of the pattern because it causes problem with
+                 * false matches against absolute paths.
+		 * If you change this to a simple ``>='' test 36 will fail,
+		 * If you change this to a simple ``>'' test 196 will fail.
+		 */
+		while
+		(
+			formal_begin > this->formal->str_text + 2
+		?
+			q >= actual_begin
+		:
+			q > actual_begin
+		)
 		{
 			this->fill[idx] =
 				str_n_from_c(actual_begin, q - actual_begin);
