@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999, 2000 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include <desist.h>
 #include <error_intl.h>
 #include <id.h>
+#include <id/global.h>
 #include <id/variable.h>
 #include <match/stack.h>
 #include <mem.h>
@@ -35,7 +36,9 @@
 #include <opcode/list.h>
 #include <opcode/thread-id.h>
 #include <option.h>
+#include <os/wait.h>
 #include <str_list.h>
+#include <symtab.h>
 #include <trace.h>
 
 
@@ -74,23 +77,22 @@ opcode_context_new(olp, mp)
 	ocp->value_stack_length = 0;
 	ocp->value_stack_maximum = 0;
 	ocp->value_stack = 0;
+	ocp->thread_id = opcode_thread_id_borrow();
+	ocp->msp = 0;
 	ocp->pid = 0;
 	ocp->exit_status = 0;
 	ocp->rup = 0;
+	ocp->wlp = 0;
+	ocp->need_age = 0;
+	ocp->flags = 0;
+	ocp->mp = 0;
+	ocp->thread_stp = 0;
+	ocp->host_binding = 0;
 	ocp->gp = 0;
 
-	ocp->target = 0;
-	ocp->targets = 0;
-	ocp->need = 0;
-	ocp->younger = 0;
-	ocp->flags = 0;
-	ocp->msp = 0;
-	ocp->host_binding = 0;
-
-	ocp->thread_id = opcode_thread_id_borrow();
-
 	opcode_context_match_push(ocp, mp);
-	opcode_context_call(ocp, olp);
+	if (olp)
+		opcode_context_call(ocp, olp);
 	trace((/*{*/"}\n"));
 	return ocp;
 }
@@ -132,18 +134,12 @@ opcode_context_delete(ocp)
 	ocp->value_stack = 0;
 	if (ocp->rup)
 		mem_free(ocp->rup);
+	if (ocp->thread_stp)
+		symtab_free(ocp->thread_stp);
 	if (ocp->msp)
 		match_stack_delete(ocp->msp);
 
 	trace(("mark\n"));
-	if (ocp->target)
-		string_list_delete(ocp->target);
-	if (ocp->targets)
-		string_list_delete(ocp->targets);
-	if (ocp->need)
-		string_list_delete(ocp->need);
-	if (ocp->younger)
-		string_list_delete(ocp->younger);
 	if (ocp->flags)
 		mem_free(ocp->flags);
 	/* do not free the match pointer, it does not belong to us */
@@ -214,6 +210,11 @@ opcode_context_execute_inner(ocp, func)
 		{
 			trace(("rtn\n"));
 			ocp->call_stack_length--;
+			if (frame->stp)
+				symtab_free(frame->stp);
+			frame->stp = 0;
+			frame->olp = 0;
+			frame->pc = 0;
 			continue;
 		}
 
@@ -299,7 +300,7 @@ opcode_context_execute_nowait(ocp)
 			{
 				int	pid;
 
-				pid = wait(&ocp->exit_status);
+				pid = os_waitpid(ocp->pid, &ocp->exit_status);
 				if (pid < 0)
 				{
 					sub_context_ty	*scp;
@@ -387,6 +388,7 @@ opcode_context_call(ocp, olp)
 	frame = &ocp->call_stack[ocp->call_stack_length++];
 	frame->olp = olp;
 	frame->pc = 0;
+	frame->stp = 0;
 	trace((/*{*/"}\n"));
 }
 
@@ -582,44 +584,15 @@ opcode_context_waited(ocp, es)
 }
 
 
-static string_list_ty *id_get _((string_ty *));
-
-static string_list_ty *
-id_get(name)
-	string_ty	*name;
-{
-	id_ty		*idp;
-	string_list_ty	*slp;
-
-	idp = id_search(name);
-	if (!idp)
-		return 0;
-	slp = string_list_new();
-	id_variable_query(idp, slp);
-	return slp;
-}
-
-
 void
 opcode_context_suspend(ocp)
 	opcode_context_ty *ocp;
 {
-	/*
-	 * save and clear variable state
-	 */
-	trace(("opcode_context_suspend(ocp = %08lX)\n{\n"/*}*/, (long)ocp));
-	ocp->target = id_get(id_target);
-	id_unassign(id_target);
-	ocp->targets = id_get(id_targets);
-	id_unassign(id_targets);
-	ocp->need = id_get(id_need);
-	id_unassign(id_need);
-	ocp->younger = id_get(id_younger);
-	id_unassign(id_younger);
 
 	/*
 	 * save and clear flag state
 	 */
+	trace(("opcode_context_suspend(ocp = %08lX)\n{\n"/*}*/, (long)ocp));
 	ocp->flags = option_flag_state_get();
 	option_undo_level(OPTION_LEVEL_EXECUTE);
 	option_undo_level(OPTION_LEVEL_RECIPE);
@@ -631,38 +604,10 @@ void
 opcode_context_resume(ocp)
 	opcode_context_ty *ocp;
 {
-	/*
-	 * restore variable state
-	 */
-	trace(("opcode_context_resume(ocp = %08lX)\n{\n"/*}*/, (long)ocp));
-	if (ocp->target)
-	{
-		id_assign_push(id_target, id_variable_new(ocp->target));
-		string_list_delete(ocp->target);
-		ocp->target = 0;
-	}
-	if (ocp->targets)
-	{
-		id_assign_push(id_targets, id_variable_new(ocp->targets));
-		string_list_delete(ocp->targets);
-		ocp->targets = 0;
-	}
-	if (ocp->need)
-	{
-		id_assign_push(id_need, id_variable_new(ocp->need));
-		string_list_delete(ocp->need);
-		ocp->need = 0;
-	}
-	if (ocp->younger)
-	{
-		id_assign_push(id_younger, id_variable_new(ocp->younger));
-		string_list_delete(ocp->younger);
-		ocp->younger = 0;
-	}
-
 	/*
 	 * restore flag state
 	 */
+	trace(("opcode_context_resume(ocp = %08lX)\n{\n"/*}*/, (long)ocp));
 	assert(ocp->flags);
 	if (ocp->flags)
 		option_flag_state_set(ocp->flags);
@@ -711,3 +656,174 @@ opcode_context_match_push(ocp, mp)
 		ocp->msp = match_stack_new();
 	match_stack_push(ocp->msp, mp);
 }
+
+
+void
+opcode_context_id_assign(ocp, name, value, local)
+	opcode_context_ty *ocp;
+	string_ty	*name;
+	id_ty		*value;
+	int		local;
+{
+	opcode_frame_ty	*frame;
+
+	assert(ocp);
+	if (ocp->call_stack_length < 1)
+		frame = 0;
+	else
+		frame = &ocp->call_stack[ocp->call_stack_length - 1];
+	if (local > 0 && frame)
+	{
+		if (!frame->stp)
+		{
+			frame->stp = symtab_alloc(5);
+			frame->stp->reap = id_global_reap;
+		}
+		symtab_assign(frame->stp, name, value);
+	}
+	else if (local < 0)
+	{
+		if (!ocp->thread_stp)
+		{
+			ocp->thread_stp = symtab_alloc(5);
+			ocp->thread_stp->reap = id_global_reap;
+		}
+		symtab_assign(ocp->thread_stp, name, value);
+	}
+	else if (frame && frame->stp && symtab_query(frame->stp, name))
+		symtab_assign(frame->stp, name, value);
+	else if (ocp->thread_stp && symtab_query(ocp->thread_stp, name))
+		symtab_assign(ocp->thread_stp, name, value);
+	else
+		symtab_assign(id_global_stp(), name, value);
+}
+
+
+id_ty *
+opcode_context_id_search(ocp, name)
+	const opcode_context_ty *ocp;
+	string_ty	*name;
+{
+	id_ty		*result;
+
+	assert(ocp);
+	if (ocp->call_stack_length >= 1)
+	{
+		opcode_frame_ty	*frame;
+
+		frame = &ocp->call_stack[ocp->call_stack_length - 1];
+		if (frame->stp)
+		{
+			result = symtab_query(frame->stp, name);
+			if (result)
+				return result;
+		}
+	}
+	if (ocp->thread_stp)
+	{
+		result = symtab_query(ocp->thread_stp, name);
+		if (result)
+			return result;
+	}
+	return symtab_query(id_global_stp(), name);
+}
+
+
+id_ty *
+opcode_context_id_search_fuzzy(ocp, name, guess)
+	const opcode_context_ty *ocp;
+	string_ty	*name;
+	string_ty	**guess;
+{
+	symtab_ty	*argv[3];
+	size_t		argc;
+
+	*guess = 0;
+	assert(ocp);
+	assert(ocp->call_stack_length > 0);
+	argc = 0;
+	if (ocp->call_stack_length >= 1)
+	{
+		opcode_frame_ty	*frame;
+
+		frame = &ocp->call_stack[ocp->call_stack_length - 1];
+		if (frame->stp)
+			argv[argc++] = frame->stp;
+	}
+	if (ocp->thread_stp)
+		argv[argc++] = ocp->thread_stp;
+	argv[argc++] = id_global_stp();
+	return symtab_query_fuzzyN(argv, argc, name, guess);
+}
+
+
+/*
+ * NAME
+ *	opcode_context_run
+ *
+ * SYNOPSIS
+ *	string_list_ty *opcode_context_run(opcode_list_ty *);
+ *
+ * DESCRIPTION
+ *	The opcode_context_run function is used to execute an opcode list
+ *	with the expectation of extracting a string list result.  Used
+ *	to evaluate the need1 and need2 ingredients.
+ *
+ * RETURNS
+ *	string_list_ty *; or NULL pointer on error
+ *
+ * CAVEAT
+ *	Use string_list_delete when you are done with it.
+ */
+
+string_list_ty *
+opcode_context_run(ocp, olp)
+	opcode_context_ty *ocp;
+	opcode_list_ty	*olp;
+{
+	opcode_status_ty status;
+
+	assert(ocp->call_stack_length == 0);
+	if (!olp)
+		return string_list_new();
+	opcode_context_call(ocp, olp);
+	status = opcode_context_execute(ocp);
+	if (status != opcode_status_success)
+		return 0;
+	return opcode_context_string_list_pop(ocp);
+}
+
+
+/*
+ * NAME
+ *	opcode_context_run_bool
+ *
+ * SYNOPSIS
+ *	string_list_ty *opcode_context_run_bool(opcode_list_ty *);
+ *
+ * DESCRIPTION
+ *	The opcode_context_run_bool function is used to execute an opcode list
+ *	with the expectation of extracting a boolean result.  Used
+ *	to evaluate recipe predicates.
+ *
+ * RETURNS
+ *	int; 0 if false, 1 if true, -1 on error
+ */
+
+int
+opcode_context_run_bool(ocp, olp)
+	opcode_context_ty *ocp;
+	opcode_list_ty	*olp;
+{
+	string_list_ty	*slp;
+	int		result;
+
+	if (!olp)
+		return 1;
+	slp = opcode_context_run(ocp, olp);
+	if (!slp)
+		return -1;
+	result = string_list_bool(slp);
+	string_list_delete(slp);
+	return result;
+}
