@@ -27,12 +27,14 @@
 #include <opcode/status.h>
 
 struct string_ty; /* existence */
+struct symtab_ty; /* existence */
 
 typedef struct opcode_frame_ty opcode_frame_ty;
 struct opcode_frame_ty
 {
 	struct opcode_list_ty *olp;
 	size_t		pc;
+	struct symtab_ty *stp;
 };
 
 typedef struct opcode_context_ty opcode_context_ty;
@@ -46,6 +48,7 @@ struct opcode_context_ty
 	struct string_list_ty **value_stack;
 	long		thread_id;
 
+	struct symtab_ty *thread_stp;
 	struct match_stack_ty *msp;
 
 	int		pid;		/* used by opcode_command */
@@ -55,10 +58,6 @@ struct opcode_context_ty
 	int		need_age;	/* used by graph_run */
 
 	/* for suspend/resume */
-	struct string_list_ty *target;
-	struct string_list_ty *targets;
-	struct string_list_ty *need;
-	struct string_list_ty *younger;
 	void		*flags;
 	struct match_ty	*mp;
 	struct string_ty *host_binding;
@@ -92,4 +91,16 @@ const struct match_ty *opcode_context_match_top _((const opcode_context_ty *));
 const struct match_ty *opcode_context_match_pop _((opcode_context_ty *));
 void opcode_context_match_push _((opcode_context_ty *, const struct match_ty *));
 
+struct id_ty *opcode_context_id_search _((const opcode_context_ty *,
+	struct string_ty *));
+struct id_ty *opcode_context_id_search_fuzzy _((const opcode_context_ty *,
+	struct string_ty *, struct string_ty **));
+void opcode_context_id_assign _((opcode_context_ty *, struct string_ty *,
+	struct id_ty *, int));
+void opcode_context_id_unassign _((opcode_context_ty *, struct string_ty *));
+
+struct string_list_ty *opcode_context_run _((opcode_context_ty *,
+	struct opcode_list_ty *));
+int opcode_context_run_bool _((opcode_context_ty *, struct opcode_list_ty *));
+
 #endif /* COOK_OPCODE_CONTEXT_H */
