@@ -115,13 +115,13 @@ execute(op, ocp)
 	 *
 	 * Use fuzzy matching to get nicer error messages.
 	 */
-	value = id_search(arg->string[0]);
+	value = opcode_context_id_search(ocp, arg->string[0]);
 	if (!value)
 	{
 		string_ty	*other;
 
 		status = opcode_status_error;
-		value = id_search_fuzzy(arg->string[0], &other);
+		value = opcode_context_id_search_fuzzy(ocp, arg->string[0], &other);
 		if (value)
 		{
 			sub_context_ty	*scp;
@@ -175,7 +175,7 @@ execute(op, ocp)
 			}
 			sub_context_delete(scp);
 			value = id_nothing_new();
-			id_assign(arg->string[0], value);
+			opcode_context_id_assign(ocp, arg->string[0], value, 0);
 		}
 	}
 
@@ -237,13 +237,13 @@ script(op, ocp)
 	 *
 	 * Use fuzzy matching to get nicer error messages.
 	 */
-	value = id_search(arg->string[0]);
+	value = opcode_context_id_search(ocp, arg->string[0]);
 	if (!value)
 	{
 		string_ty	*other;
 
 		status = opcode_status_error;
-		value = id_search_fuzzy(arg->string[0], &other);
+		value = opcode_context_id_search_fuzzy(ocp, arg->string[0], &other);
 		if (value)
 		{
 			sub_context_ty	*scp;
@@ -297,7 +297,7 @@ script(op, ocp)
 			}
 			sub_context_delete(scp);
 			value = id_nothing_new();
-			id_assign(arg->string[0], value);
+			opcode_context_id_assign(ocp, arg->string[0], value, 0);
 		}
 	}
 
