@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997 Peter Miller;
+ *	Copyright (C) 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -64,10 +64,22 @@ execute(op, icp)
 	slp = opcode_context_string_list_pop(icp);
 	/* first argument is the name of the function */
 	if (slp->nstrings > 0)
+	{
+		string_ty	*name;
+		string_list_ty	value;
+
+		name = str_from_c("__FUNCTION__");
+		string_list_constructor(&value);
+		string_list_append(&value, slp->string[0]);
+		opcode_context_id_assign(icp, name, id_variable_new(&value), 1);
+		str_free(name);
+		string_list_destructor(&value);
+
 		string_list_remove(slp, slp->string[0]);
+	}
 	if (!arg)
 		arg = str_from_c("arg");
-	id_assign_push(arg, id_variable_new(slp));
+	opcode_context_id_assign(icp, arg, id_variable_new(slp), 1);
 	for (j = 1; j <= 9; ++j)
 	{
 		string_ty	*name;
@@ -77,7 +89,7 @@ execute(op, icp)
 		string_list_constructor(&value);
 		if (j <= slp->nstrings)
 			string_list_append(&value, slp->string[j - 1]);
-		id_assign_push(name, id_variable_new(&value));
+		opcode_context_id_assign(icp, name, id_variable_new(&value), 1);
 		str_free(name);
 		string_list_destructor(&value);
 	}
