@@ -169,6 +169,60 @@ adjust_message(path, nsec)
 	trace(("}\n"));
 }
 
+#ifdef __CYGWIN32__
+
+/*
+ * NAME
+ *	utimes4cygwin
+ *
+ * DESCRIPTION
+ *	On Cygwin, some files magically get an .exe ending to the
+ *	filename.  Therefore, we insert an extra check after all other
+ *	handling has been done to check for that case.	This fix needs to
+ *	be here because os_exists(filename) returns true if "filename"
+ *	or "filename.exe" exist, but utime(filename) only works with
+ *	"filename". So we do this here, after we know that there is no
+ *	file called "filename".
+ *
+ * RETURNS
+ *	int; 0 on success, -1 and errno on failure.
+ *
+ * AUTHOR
+ *	This code was contributed by Adam Schlegel <aceschle@thinkage.ca>
+ */
+
+static int utimes4cygwin _((string_ty *path, struct utimbuf *ut));
+
+static int
+utimes4cygwin(path, ut)
+	string_ty	*path;
+	struct utimbuf	*ut;
+{
+	string_ty	*fullpath;
+	int		j;
+	const char	*suffix;
+	int		err;
+
+	for (j = 0; ; j++)
+	{
+		suffix = exeext_nth(j);
+		if (!suffix || !*suffix)
+		{
+			errno = ENOENT;
+			return -1;
+		}
+		fullpath = str_catenate(path, str_from_c(suffix));
+		err = utime(fullpath->str_text, ut);
+		if (!err || errno != ENOENT)
+		{
+			str_free(fullpath);
+			return err;
+		}
+		str_free(fullpath);
+	}	
+}
+
+#endif
 
 /*
  * NAME
@@ -225,7 +279,17 @@ os_mtime_adjust(path, min_age)
 				ut.actime = min_age;
 				err = utime(path->str_text, &ut);
 				if (err && errno == ENOENT)
+				{
+					errno = 0;
 					err = archive_utime(path, &ut);
+				}
+#ifdef __CYGWIN32__
+				if (err && errno == ENOENT)
+				{
+					errno = 0;
+					err = utimes4cygwin(path, &ut);
+				}
+#endif /* __CYGWIN32__ */
 				if (err && errno == EPERM)
 				{
 					sub_context_ty	*scp;
