@@ -69,7 +69,7 @@ case $file in
 
 	echo ""
 	echo "${stem}.gen.\$(OBJEXT): ${stem}.gen.c" $dep
-	echo "	\$(CC) \$(CPPFLAGS) \$(CFLAGS) -I$dir -Icommon -c ${stem}.gen.c"
+	echo "	\$(CC) -I$dir -Icommon \$(CPPFLAGS) \$(CFLAGS) -c ${stem}.gen.c"
 	echo "	mv ${root}.gen.\$(OBJEXT) \$@"
 	;;
 
@@ -84,7 +84,7 @@ case $file in
 
 	echo ""
 	echo "${stem}.\$(OBJEXT): $file" $dep
-	echo "	\$(CC) \$(CPPFLAGS) \$(CFLAGS) -I$dir -Icommon -c $file"
+	echo "	\$(CC) -I$dir -Icommon \$(CPPFLAGS) \$(CFLAGS) -c $file"
 	echo "	mv ${root}.\$(OBJEXT) \$@"
 	;;
 
