@@ -438,7 +438,7 @@ gram_lex()
 	int		first_was;
 	string_ty	*s;
 	int		token;
-	int		start_of_line;
+	int		start_of_line = 0;
 
 	trace(("gram_lex()\n{\n"/*}*/));
 	for (;;)
