@@ -67,15 +67,19 @@ cat > book << 'fubar'
 cc = [getenv CC]; if [not [cc]] then
 { cc = [find_command gcc]; if [not [cc]] then cc = cc; }
 
+exe = ;
+if [in [substr 1 6 [os]] CYGWIN NUTCRA] then
+	exe = .exe;
+
 search_list  = . bl;
 %.o: %.c
 {
 	[cc] -O -c [resolve %.c];
 }
 
-test: a.o b.o
+test[exe]: a.o b.o
 {
-	[cc] -o test [resolve a.o b.o];
+	[cc] -o test[exe] [resolve a.o b.o];
 }
 fubar
 if test $? -ne 0 ; then fail; fi
@@ -109,19 +113,19 @@ if test $? -ne 0 ; then fail; fi
 # try it out
 #
 sleep 1
-$bin/cook -book book -nl > /dev/null 2>&1
-if test $? -ne 0 ; then fail; fi
+$bin/cook -book book -nl > LOG 2>&1
+if test $? -ne 0 ; then cat LOG; fail; fi
 
 #
 # make sure works for object in search list, too
 #
 mv b.o bl/b.o
 if test $? -ne 0 ; then fail; fi
-rm test
+rm test > LOG 2>&1 || rm test.exe > LOG 2>&1
 if test $? -ne 0 ; then fail; fi
 sleep 1
-$bin/cook -book book -nl > /dev/null 2>&1
-if test $? -ne 0 ; then fail; fi
+$bin/cook -book book -nl > LOG 2>&1
+if test $? -ne 0 ; then cat LOG; fail; fi
 
 #
 # Only definite negatives are possible.
