@@ -79,7 +79,10 @@ test: a.id b.id
 
 %.id:
 {
-	echo [thread-id] > [target];
+	/* You need the sleep here, otherwise the test will fail on fast
+	   machines, because the first target finishes and comes back
+	   before cook can launch the second one. */
+	sleep 1 && echo [thread-id] > [target];
 }
 fubar
 if test $? -ne 0 ; then no_result; fi
